package Gamla::Web::PathMan::Pointer::WithMount;

use strict;

use vars qw(@ISA);

use Gamla::Object;

@ISA=qw(Gamla::Object);

sub init_scalars
{
    my $self = shift;

    my $params = shift;

    my $pathman = shift(@{$params});
    $self->{'path_man'} = $pathman;

    $self->{'cur_dir'} = ($pathman->get_root_dir());

    $self->{'pm_path'} = [];

    $self->{'is_in_mounted'} = 0;

    $self->handle_mounting();

    return 0;
}

sub init_options
{
    my $self = shift;

    $self->{'recursive_mount'} = 0;
    $self->{'recursive_write'} = 1;

    my ($opt, @ret);

    while ($opt = shift)
    {
        if ($opt =~ /^-?recursive(_|-)mount$/i)
        {
            $self->{'recursive_mount'} = (!!shift);
        }
        elsif ($opt =~ /^-?recursive(_|-)write$/i)
        {
            $self->{'recursive_write'} = (!!shift);
        }
        else
        {
            push @ret, $opt, shift(@_);
        }
    }

    if (! $self->{'recursive_write'} )
    {
        $self->{'recursive_mount'} = 0;
    }

    return @ret;
}

sub initialize
{
    my $self = shift;

    $self->init_scalars(\@_);

    $self->init_options(@_);

    return 0;
}

sub handle_mounting
{
    my $self = shift;

    if ($self->is_mounting_point())
    {
        my $mounted_pointer = $self->get_mounted_pathman_pointer();
        $mounted_pointer->cd($self->get_mounted_pathman_path_to_start_from());
        $self->{'mounted_pointer'} = $mounted_pointer;
        $self->{'is_in_mounted'} = 1;
        $self->{'len_orig_path'} = scalar(@{$self->{'pm_path'}});
    }
}

sub mkdir_
{
    my $self = shift;

    my $subdir_name = shift;

    if ($self->{'is_in_mounted'})
    {
        if ($self->{'recursive_write'})
        {
            return $self->{'mounted_pointer'}->mkdir_($subdir_name);
        }
        else
        {
            return (1, "No recursive writes are allowed in mounted pathmans");
        }
    }
    else
    {
        if ($self->dir_exists($subdir_name))
        {
            return (2, "There's already a directory by that name");
        }
        else
        {
            $self->local_mkdir($subdir_name);
        }
    }
}
